@@ -52,6 +52,20 @@ class BaseValueManager(object):
         else:
             return property(_get, _set)
 
+class ValueQuoter(object):
+    """
+    contains a single value, which will quote itself for CQL insertion statements
+    """
+    def __init__(self, value):
+        self.value = value
+
+    def __str__(self):
+        raise NotImplementedError
+
+    def __repr__(self):
+        return self.__str__()
+
+
 class Column(object):
 
     #the cassandra type this column maps to
@@ -60,7 +74,7 @@ class Column(object):
 
     instance_counter = 0
 
-    def __init__(self, primary_key=False, index=False, db_field=None, default=None, required=True):
+    def __init__(self, primary_key=False, partition_key=False, index=False, db_field=None, default=None, required=True, clustering_order=None):
         """
         :param primary_key: bool flag, indicates this column is a primary key. The first primary key defined
         on a model is the partition key, all others are cluster keys
@@ -69,15 +83,13 @@ class Column(object):
         :param default: the default value, can be a value or a callable (no args)
         :param required: boolean, is the field required?
         """
-        self.primary_key = primary_key
+        self.partition_key = partition_key
+        self.primary_key = partition_key or primary_key
         self.index = index
         self.db_field = db_field
         self.default = default
         self.required = required
-
-        #only the model meta class should touch this
-        self._partition_key = False
-
+        self.clustering_order = clustering_order
         #the column name in the model definition
         self.column_name = None
 
@@ -137,7 +149,7 @@ class Column(object):
         """
         Returns a column definition for CQL table definition
         """
-        return '"{}" {}'.format(self.db_field_name, self.db_type)
+        return '{} {}'.format(self.cql, self.db_type)
 
     def set_column_name(self, name):
         """
@@ -156,6 +168,13 @@ class Column(object):
         """ Returns the name of the cql index """
         return 'index_{}'.format(self.db_field_name)
 
+    @property
+    def cql(self):
+        return self.get_cql()
+
+    def get_cql(self):
+        return '"{}"'.format(self.db_field_name)
+
 class Bytes(Column):
     db_type = 'blob'
 
@@ -183,6 +202,7 @@ class Text(Column):
                 raise ValidationError('{} is shorter than {} characters'.format(self.column_name, self.min_length))
         return value
 
+
 class Integer(Column):
     db_type = 'int'
 
@@ -200,22 +220,34 @@ class Integer(Column):
     def to_database(self, value):
         return self.validate(value)
 
+
 class DateTime(Column):
     db_type = 'timestamp'
+
     def __init__(self, **kwargs):
         super(DateTime, self).__init__(**kwargs)
 
     def to_python(self, value):
         if isinstance(value, datetime):
             return value
+        elif isinstance(value, date):
+            return datetime(*(value.timetuple()[:6]))
         return datetime.utcfromtimestamp(value)
 
     def to_database(self, value):
         value = super(DateTime, self).to_database(value)
+        if value is None: return
         if not isinstance(value, datetime):
-            raise ValidationError("'{}' is not a datetime object".format(value))
-        epoch = datetime(1970, 1, 1)
-        return long((value - epoch).total_seconds() * 1000)
+            if isinstance(value, date):
+                value = datetime(value.year, value.month, value.day)
+            else:
+                raise ValidationError("'{}' is not a datetime object".format(value))
+        epoch = datetime(1970, 1, 1, tzinfo=value.tzinfo)
+        offset = 0
+        if epoch.tzinfo:
+            offset_delta = epoch.tzinfo.utcoffset(epoch)
+            offset = offset_delta.days*24*3600 + offset_delta.seconds
+        return long(((value  - epoch).total_seconds() - offset) * 1000)
 
 
 class Date(Column):
@@ -276,11 +308,16 @@ class TimeUUID(UUID):
 class Boolean(Column):
     db_type = 'boolean'
 
+    class Quoter(ValueQuoter):
+        """ Cassandra 1.2.5 is stricter about boolean values """
+        def __str__(self):
+            return 'true' if self.value else 'false'
+
     def to_python(self, value):
         return bool(value)
 
     def to_database(self, value):
-        return bool(value)
+        return self.Quoter(bool(value))
 
 class Float(Column):
     db_type = 'double'
@@ -312,16 +349,6 @@ class Counter(Column):
         super(Counter, self).__init__(**kwargs)
         raise NotImplementedError
 
-class ContainerQuoter(object):
-    """
-    contains a single value, which will quote itself for CQL insertion statements
-    """
-    def __init__(self, value):
-        self.value = value
-
-    def __str__(self):
-        raise NotImplementedError
-
 class BaseContainerColumn(Column):
     """
     Base Container type
@@ -363,7 +390,7 @@ class Set(BaseContainerColumn):
     """
     db_type = 'set<{}>'
 
-    class Quoter(ContainerQuoter):
+    class Quoter(ValueQuoter):
 
         def __str__(self):
             cq = cql_quote
@@ -446,7 +473,7 @@ class List(BaseContainerColumn):
     """
     db_type = 'list<{}>'
 
-    class Quoter(ContainerQuoter):
+    class Quoter(ValueQuoter):
 
         def __str__(self):
             cq = cql_quote
@@ -543,7 +570,7 @@ class Map(BaseContainerColumn):
 
     db_type = 'map<{}, {}>'
 
-    class Quoter(ContainerQuoter):
+    class Quoter(ValueQuoter):
 
         def __str__(self):
             cq = cql_quote
@@ -600,6 +627,8 @@ class Map(BaseContainerColumn):
         prev = self.to_database(prev)
         if isinstance(val, self.Quoter): val = val.value
         if isinstance(prev, self.Quoter): prev = prev.value
+        val = val or {}
+        prev = prev or {}
 
         #get the updated map
         update = {k:v for k,v in val.items() if v != prev.get(k)}
@@ -627,8 +656,8 @@ class Map(BaseContainerColumn):
         if isinstance(val, self.Quoter): val = val.value
         if isinstance(prev, self.Quoter): prev = prev.value
 
-        old_keys = set(prev.keys())
-        new_keys = set(val.keys())
+        old_keys = set(prev.keys()) if prev else set()
+        new_keys = set(val.keys()) if val else set()
         del_keys = old_keys - new_keys
 
         del_statements = []
@@ -639,4 +668,19 @@ class Map(BaseContainerColumn):
 
         return del_statements
 
+class _PartitionKeysToken(Column):
+    """
+    virtual column representing token of partition columns.
+    Used by filter(pk__token=Token(...)) filters
+    """
+
+    def __init__(self, model):
+        self.partition_columns = model._partition_keys.values()
+        super(_PartitionKeysToken, self).__init__(partition_key=True)
+
+    def to_database(self, value):
+        raise NotImplementedError
+
+    def get_cql(self):
+        return "token({})".format(", ".join(c.cql for c in self.partition_columns))
 
