@@ -14,6 +14,7 @@ def create_keyspace(name, strategy_class='SimpleStrategy', replication_factor=3,
     :param **replication_values: 1.2 only, additional values to ad to the replication data map
     """
     with connection_manager() as con:
+        #TODO: check system tables instead of using cql thrifteries
         if not any([name == k.name for k in con.con.client.describe_keyspaces()]):
 #        if name not in [k.name for k in con.con.client.describe_keyspaces()]:
             try:
@@ -55,26 +56,36 @@ def create_table(model, create_missing_keyspace=True):
 
     with connection_manager() as con:
         #check for an existing column family
+        #TODO: check system tables instead of using cql thrifteries
         ks_info = con.con.client.describe_keyspace(model._get_keyspace())
         if not any([raw_cf_name == cf.name for cf in ks_info.cf_defs]):
             qs = ['CREATE TABLE {}'.format(cf_name)]
 
             #add column types
             pkeys = []
+            ckeys = []
             qtypes = []
             def add_column(col):
                 s = col.get_column_def()
-                if col.primary_key: pkeys.append('"{}"'.format(col.db_field_name))
+                if col.primary_key:
+                    keys = (pkeys if col.partition_key else ckeys)
+                    keys.append('"{}"'.format(col.db_field_name))
                 qtypes.append(s)
             for name, col in model._columns.items():
                 add_column(col)
 
-            qtypes.append('PRIMARY KEY ({})'.format(', '.join(pkeys)))
+            qtypes.append('PRIMARY KEY (({}){})'.format(', '.join(pkeys), ckeys and ', ' + ', '.join(ckeys) or ''))
             
             qs += ['({})'.format(', '.join(qtypes))]
-            
+
+            with_qs = ['read_repair_chance = {}'.format(model.read_repair_chance)]
+
+            _order = ["%s %s" % (c.db_field_name, c.clustering_order or 'ASC') for c in model._clustering_keys.values()]
+            if _order:
+                with_qs.append("clustering order by ({})".format(', '.join(_order)))
+
             # add read_repair_chance
-            qs += ['WITH read_repair_chance = {}'.format(model.read_repair_chance)]
+            qs += ['WITH {}'.format(' AND '.join(with_qs))]
             qs = ' '.join(qs)
 
             try:
