@@ -4,16 +4,20 @@ from datetime import datetime
 from hashlib import md5
 from time import time
 from uuid import uuid1
-from cqlengine import BaseContainerColumn, BaseValueManager, Map
+from cqlengine import BaseContainerColumn, BaseValueManager, Map, columns
 
 from cqlengine.connection import connection_manager
 from cqlengine.exceptions import CQLEngineException
-from cqlengine.functions import BaseQueryFunction
+from cqlengine.functions import QueryValue, Token
 
 #CQL 3 reference:
 #http://www.datastax.com/docs/1.1/references/cql/index
 
 class QueryException(CQLEngineException): pass
+class DoesNotExist(QueryException): pass
+class MultipleObjectsReturned(QueryException): pass
+
+
 class QueryOperatorException(QueryException): pass
 
 class QueryOperator(object):
@@ -24,14 +28,16 @@ class QueryOperator(object):
     # The comparator symbol this operator uses in cql
     cql_symbol = None
 
+    QUERY_VALUE_WRAPPER = QueryValue
+
     def __init__(self, column, value):
         self.column = column
         self.value = value
 
-        #the identifier is a unique key that will be used in string
-        #replacement on query strings, it's created from a hash
-        #of this object's id and the time
-        self.identifier = uuid1().hex
+        if isinstance(value, QueryValue):
+            self.query_value = value
+        else:
+            self.query_value = self.QUERY_VALUE_WRAPPER(value)
 
         #perform validation on this operator
         self.validate_operator()
@@ -41,12 +47,8 @@ class QueryOperator(object):
     def cql(self):
         """
         Returns this operator's portion of the WHERE clause
-        :param valname: the dict key that this operator's compare value will be found in
         """
-        if isinstance(self.value, BaseQueryFunction):
-            return '"{}" {} {}'.format(self.column.db_field_name, self.cql_symbol, self.value.to_cql(self.identifier))
-        else:
-            return '"{}" {} :{}'.format(self.column.db_field_name, self.cql_symbol, self.identifier)
+        return '{} {} {}'.format(self.column.cql, self.cql_symbol, self.query_value.cql)
 
     def validate_operator(self):
         """
@@ -81,10 +83,7 @@ class QueryOperator(object):
         this should return the dict: {'colval':<self.value>}
         SELECT * FROM column_family WHERE colname=:colval
         """
-        if isinstance(self.value, BaseQueryFunction):
-            return {self.identifier: self.column.to_database(self.value.get_value())}
-        else:
-            return {self.identifier: self.column.to_database(self.value)}
+        return self.query_value.get_dict(self.column)
 
     @classmethod
     def get_operator(cls, symbol):
@@ -102,34 +101,41 @@ class QueryOperator(object):
         except KeyError:
             raise QueryOperatorException("{} doesn't map to a QueryOperator".format(symbol))
 
+    # equality operator, used by tests
+
+    def __eq__(self, op):
+        return self.__class__ is op.__class__ and \
+                self.column.db_field_name == op.column.db_field_name and \
+                self.value == op.value
+
+    def __ne__(self, op):
+        return not (self == op)
+
+    def __hash__(self):
+        return hash(self.column.db_field_name) ^ hash(self.value)
+
 class EqualsOperator(QueryOperator):
     symbol = 'EQ'
     cql_symbol = '='
 
+class IterableQueryValue(QueryValue):
+    def __init__(self, value):
+        try:
+            super(IterableQueryValue, self).__init__(value, [uuid1().hex for i in value])
+        except TypeError:
+            raise QueryException("in operator arguments must be iterable, {} found".format(value))
+
+    def get_dict(self, column):
+        return dict((i, column.to_database(v)) for (i, v) in zip(self.identifier, self.value))
+
+    def get_cql(self):
+        return '({})'.format(', '.join(':{}'.format(i) for i in self.identifier))
+
 class InOperator(EqualsOperator):
     symbol = 'IN'
     cql_symbol = 'IN'
 
-    class Quoter(object):
-        """
-        contains a single value, which will quote itself for CQL insertion statements
-        """
-        def __init__(self, value):
-            self.value = value
-
-        def __str__(self):
-            from cql.query import cql_quote as cq
-            return '(' + ', '.join([cq(v) for v in self.value]) + ')'
-
-    def get_dict(self):
-        if isinstance(self.value, BaseQueryFunction):
-            return {self.identifier: self.column.to_database(self.value.get_value())}
-        else:
-            try:
-                values = [v for v in self.value]
-            except TypeError:
-                raise QueryException("in operator arguments must be iterable, {} found".format(self.value))
-            return {self.identifier: self.Quoter([self.column.to_database(v) for v in self.value])}
+    QUERY_VALUE_WRAPPER = IterableQueryValue
 
 class GreaterThanOperator(QueryOperator):
     symbol = "GT"
@@ -222,6 +228,9 @@ class QuerySet(object):
         self._defer_fields = []
         self._only_fields = []
 
+        self._values_list = False
+        self._flat_values_list = False
+
         #results cache
         self._con = None
         self._cur = None
@@ -260,7 +269,8 @@ class QuerySet(object):
         return clone
 
     def __len__(self):
-        return self.count()
+        self._execute_query()
+        return len(self._result_cache)
 
     def __del__(self):
         if self._con:
@@ -275,14 +285,17 @@ class QuerySet(object):
 
         #check that there's either a = or IN relationship with a primary key or indexed field
         equal_ops = [w for w in self._where if isinstance(w, EqualsOperator)]
-        if not any([w.column.primary_key or w.column.index for w in equal_ops]):
+        token_ops = [w for w in self._where if isinstance(w.value, Token)]
+        if not any([w.column.primary_key or w.column.index for w in equal_ops]) and not token_ops:
             raise QueryException('Where clauses require either a "=" or "IN" comparison with either a primary key or indexed field')
 
         if not self._allow_filtering:
             #if the query is not on an indexed field
             if not any([w.column.index for w in equal_ops]):
-                if not any([w.column._partition_key for w in equal_ops]):
+                if not any([w.column.partition_key for w in equal_ops]) and not token_ops:
                     raise QueryException('Filtering on a clustering key without a partition key is not allowed unless allow_filtering() is called on the querset')
+            if any(not w.column.partition_key for w in token_ops):
+                raise QueryException('The token() function is only supported on the partition key')
 
 
         #TODO: abuse this to see if we can get cql to raise an exception
@@ -307,7 +320,7 @@ class QuerySet(object):
         if self._defer_fields:
             fields = [f for f in fields if f not in self._defer_fields]
         elif self._only_fields:
-            fields = [f for f in fields if f in self._only_fields]
+            fields = self._only_fields
         db_fields = [self.model._columns[f].db_field_name for f in fields]
 
         qs = ['SELECT {}'.format(', '.join(['"{}"'.format(f) for f in db_fields]))]
@@ -336,6 +349,9 @@ class QuerySet(object):
             self._con = connection_manager()
             self._cur = self._con.execute(self._select_query(), self._where_values())
             self._result_cache = [None]*self._cur.rowcount
+            if self._cur.description:
+                names = [i[0] for i in self._cur.description]
+                self._construct_result = self._create_result_constructor(names)
 
     def _fill_result_cache_to_idx(self, idx):
         self._execute_query()
@@ -346,14 +362,12 @@ class QuerySet(object):
         if qty < 1:
             return
         else:
-            names = [i[0] for i in self._cur.description]
             for values in self._cur.fetchmany(qty):
-                value_dict = dict(zip(names, values))
                 self._result_idx += 1
-                self._result_cache[self._result_idx] = self._construct_instance(value_dict)
+                self._result_cache[self._result_idx] = self._construct_result(values)
 
             #return the connection to the connection pool if we have all objects
-            if self._result_cache and self._result_cache[-1] is not None:
+            if self._result_cache and self._result_idx == (len(self._result_cache) - 1):
                 self._con.close()
                 self._con = None
                 self._cur = None
@@ -394,19 +408,26 @@ class QuerySet(object):
                 self._fill_result_cache_to_idx(s)
                 return self._result_cache[s]
 
+    def _create_result_constructor(self, names):
+        """
+        Returns a function that will be used to instantiate query results
+        """
+        model = self.model
+        db_map = model._db_map
+        if not self._values_list:
+            def _construct_instance(values):
+                field_dict = dict((db_map.get(k, k), v) for k, v in zip(names, values))
+                instance = model(**field_dict)
+                instance._is_persisted = True
+                return instance
+            return _construct_instance
 
-    def _construct_instance(self, values):
-        #translate column names to model names
-        field_dict = {}
-        db_map = self.model._db_map
-        for key, val in values.items():
-            if key in db_map:
-                field_dict[db_map[key]] = val
-            else:
-                field_dict[key] = val
-        instance = self.model(**field_dict)
-        instance._is_persisted = True
-        return instance
+        columns = [model._columns[db_map[name]] for name in names]
+        if self._flat_values_list:
+           return (lambda values: columns[0].to_python(values[0]))
+        else:
+            # result_cls = namedtuple("{}Tuple".format(self.model.__name__), names)
+            return (lambda values: map(lambda (c, v): c.to_python(v), zip(columns, values)))
 
     def batch(self, batch_obj):
         """
@@ -427,9 +448,7 @@ class QuerySet(object):
             return None
 
     def all(self):
-        clone = copy.deepcopy(self)
-        clone._where = []
-        return clone
+        return copy.deepcopy(self)
 
     def _parse_filter_arg(self, arg):
         """
@@ -437,7 +456,7 @@ class QuerySet(object):
         <colname>__<op>
         :returns: colname, op tuple
         """
-        statement = arg.split('__')
+        statement = arg.rsplit('__', 1)
         if len(statement) == 1:
             return arg, None
         elif len(statement) == 2:
@@ -454,7 +473,10 @@ class QuerySet(object):
             try:
                 column = self.model._columns[col_name]
             except KeyError:
-                raise QueryException("Can't resolve column name: '{}'".format(col_name))
+                if col_name == 'pk__token':
+                    column = columns._PartitionKeysToken(self.model)
+                else:
+                    raise QueryException("Can't resolve column name: '{}'".format(col_name))
 
             #get query operator, or use equals if not supplied
             operator_class = QueryOperator.get_operator(col_op or 'EQ')
@@ -616,6 +638,25 @@ class QuerySet(object):
             with connection_manager() as con:
                 con.execute(qs, self._where_values())
 
+    def values_list(self, *fields, **kwargs):
+        """ Instructs the query set to return tuples, not model instance """
+        flat = kwargs.pop('flat', False)
+        if kwargs:
+            raise TypeError('Unexpected keyword arguments to values_list: %s'
+                    % (kwargs.keys(),))
+        if flat and len(fields) > 1:
+            raise TypeError("'flat' is not valid when values_list is called with more than one field.")
+        clone = self.only(fields)
+        clone._values_list = True
+        clone._flat_values_list = flat
+        return clone
+
+    def __eq__(self, q):
+        return set(self._where) == set(q._where)
+
+    def __ne__(self, q):
+        return not (self != q)
+
 class DMLQuery(object):
     """
     A query object used for queries performing inserts, updates, or deletes
